;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Guix package for development.
;;

;;; Code:
(use-modules (guix)
             (gnu packages autotools)
             (gnu packages gettext)
             (gnu packages guile)
             (gnu packages pkg-config)
	     (gnu packages perl)
             (gnu packages version-control)
	     (guix build-system gnu)
	     (guix download)
	     (guix packages)
	     ((guix licenses) #:prefix license:))

(define (guile-gettextpo-package guile)
  (let ((var (module-variable (current-module) 'guile-gettextpo)))
    (if var
	(variable-ref var)
	(let ((url "https://savannah.nongnu.org/guile-gettextpo"))
	  (package
	    (name "guile-gettextpo")
	    (version "0.1")
	    (source
	     (origin
	       (method url-fetch)
	       (uri (string-append url "/download/" name "-"
				   version ".tar.gz"))
	       (sha256
		(base32
		 "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"))))
	    (build-system gnu-build-system)
	    (native-inputs
	     `(("pkg-config" ,pkg-config)))
	    (inputs
	     `(("gettext" ,gnu-gettext)
	       ("guile" ,guile)))

	    (home-page url)
	    (synopsis "Guile bindings for libgettextpo")
	    (description
	     "Guile-gettextpo is a Guile wrapper for GNU Gettext's
libgettextpo.  It provides functions to create, manipulate, extract
and write PO files.")
	    (license license:gpl3+))))))

(let* ((guile guile-2.2)
       (guile-gettextpo (guile-gettextpo-package guile)))
  (package
    (inherit guile-gettextpo)
    ;; Development tools
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ("libtool" ,libtool)
       ("git" ,git)
       ("perl" ,perl)
       ,@(package-native-inputs guile-gettextpo)))))
