;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; External API for Guile-gettextpo
;;

;;; Code:
(define-module (gettext po)
  #:use-module (gettext po config)
  #:use-module (gettext po check)
  #:use-module (gettext po error)
  #:use-module (gettext po file)
  #:use-module (gettext po filepos)
  #:use-module (gettext po format)
  #:use-module (gettext po header)
  #:use-module (gettext po iterator)
  #:use-module (gettext po message)
  #:export (gettext-po-version
	    with-po-files))

(define gettext-po-version
  (string-append %libgettextpo-guile-name " " %libgettextpo-guile-version))

;; Export API
(let ((iface (module-public-interface (current-module)))
      (api '((gettext po check)
	     (gettext po error)
	     (gettext po file)
	     (gettext po filepos)
	     (gettext po format)
	     (gettext po header)
	     (gettext po iterator)
	     (gettext po message))))
  (for-each (lambda (name)
	      (module-use! iface (resolve-interface name)))
	    api))

(define-syntax with-po-files
  (syntax-rules ()
    ((_ ((name file) . args) body body* ...)
     (let ((name (read-po-file file)))
       (with-po-files args body body* ...)
       (write-po-file name file)))
    ((_ ((name file out-file) . args) body body* ...)
     (let ((name (read-po-file file)))
       (with-po-files args body body* ...)
       (write-po-file name out-file)))
    ((_ () body body* ...)
     (begin body body* ...))
    ((_ . args)
     (syntax-error "Unexpected args" args))))
