;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Scheme API for checking API from libgettextpo.
;;

;;; Code:
(define-module (gettext po check)
  #:use-module (gettext po error)
  #:use-module (gettext po file)
  #:use-module (gettext po message)
  #:use-module (gettext po iterator)
  #:use-module (gettext po internal-api)
  #:use-module (system foreign)
  #:export (po-format-list
	    po-format-pretty-name

	    po-file-check-all
	    po-message-check-all
	    po-message-check-format))

(define (po-format-list)
  (reverse (%po-format-list)))

(define (po-format-pretty-name format)
  (let ((str (%po-format-pretty-name (string->pointer format))))
    (and str (not (null-pointer? str))
	 (pointer->string str))))

(define* (po-file-check-all po-file #:key (error-handler #f)
			    (exception-handler #f))
  (let ((pointer (po-file-pointer po-file)))
    (call-with-xerror-handler %po-file-check-all (list pointer)
			      #:error-handler error-handler
			      #:exception-handler exception-handler)))

(define* (po-message-check-all message iterator #:key (error-handler #f)
			       (exception-handler #f))
  (let ((pointer (po-message-pointer message))
	(iterator-pointer (po-message-iterator-pointer iterator)))
    (call-with-xerror-handler %po-message-check-all
			      (list pointer iterator-pointer)
			      #:error-handler error-handler
			      #:exception-handler exception-handler)))

(define* (po-message-check-format message
				  #:key (error-handler #f)
				  (exception-handler #f))
  (let ((pointer (po-message-pointer message)))
    (call-with-xerror-handler %po-message-check-format (list pointer)
			      #:error-handler error-handler
			      #:exception-handler exception-handler)))
