;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Scheme API for po_file_t from libgettextpo.
;;

;;; Code:
(define-module (gettext po file)
  #:use-module (gettext po error)
  #:use-module (gettext po internal-api)
  #:use-module (srfi srfi-9)
  #:use-module (system foreign)
  #:export (<po-file>
	    po-file?
	    po-file-filename
	    po-file-pointer
	    po-file-eqv?

	    make-po-file
	    read-po-file
	    write-po-file
	    po-file-domains))

(define-record-type <po-file>
  (make-po-file-object filename pointer)
  po-file?
  (filename po-file-filename)
  (pointer po-file-pointer))

(define (po-file-eqv? lhs rhs)
  "Pointer comparison for <po-file> objects."
  (let ((lhs-pointer (po-file-pointer lhs))
	(rhs-pointer (po-file-pointer rhs)))
    (eqv? (pointer-address lhs-pointer) (pointer-address rhs-pointer))))

(define (make-po-file)
  "Create a new PO file, wrap"
  (let ((ptr (%po-file-create)))
    (when (or (not ptr) (null-pointer? ptr))
      ;; XXX
      (throw 'out-of-memory "Unable to create po file"))
    (let ((wrapped (make-pointer (pointer-address ptr) %po-file-free)))
      (make-po-file-object #f wrapped))))

(define* (read-po-file filename #:key (error-handler #f)
		       (exception-handler #f))
  (let ((pointer (call-with-xerror-handler %po-file-read (list filename)
					   #:error-handler error-handler
					   #:exception-handler
					   exception-handler)))
    (and pointer (not (null-pointer? pointer))
	 (make-po-file-object filename pointer))))

(define* (write-po-file po-file filename #:key (error-handler #f)
			(exception-handler #f))
  (let* ((pointer (po-file-pointer po-file))
	 (result (call-with-xerror-handler %po-file-write
					   (list pointer filename)
					   #:error-handler error-handler
					   #:exception-handler
					   exception-handler)))
    (and result (make-po-file-object filename pointer))))

(define (po-file-domains po-file)
  "The list of domains in PO-FILE or #f in case of error."
  (let ((pointer (po-file-pointer po-file)))
    (%po-file-domains pointer)))
