;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Wrapper types for C objects from/for libgettextpo.
;;

;;; Code:
(define-module (gettext po filepos)
  #:use-module (gettext po message)
  #:use-module (gettext po internal-api)
  #:use-module (srfi srfi-9)
  #:use-module (system foreign)
  #:export (<po-filepos>
	    po-filepos?
	    po-filepos-message
	    po-filepos-pointer
	    po-filepos-eqv?

	    po-filepos-file
	    po-filepos-start-line

	    po-message-filepos
	    po-message-all-filepos
	    po-message-remove-filepos!
	    po-message-add-filepos!))

(define-record-type <po-filepos>
  (make-po-filepos-object message pointer)
  po-filepos?
  (message po-filepos-message)
  (pointer po-filepos-pointer))

(define (po-filepos-eqv? lhs rhs)
  "Pointer comparison for <po-filepos> objects."
  (let ((lhs-pointer (po-filepos-pointer lhs))
	(rhs-pointer (po-filepos-pointer rhs)))
    (eqv? (pointer-address lhs-pointer) (pointer-address rhs-pointer))))

(define (po-filepos-file filepos)
  (let* ((pointer (po-filepos-pointer filepos))
	 (str (%po-filepos-file pointer)))
    (and str (not (null-pointer? str))
	 (pointer->string str))))

(define (po-filepos-start-line filepos)
  (let ((pointer (po-filepos-pointer filepos)))
    (%po-filepos-start-line pointer)))

;; Message API
(define (po-message-filepos message position)
  (let* ((pointer (po-message-pointer message))
	 (filepos-ptr (%po-message-filepos pointer position)))
    (and filepos-ptr (not (null-pointer? filepos-ptr))
	 (make-po-filepos-object message filepos-ptr))))

(define (po-message-all-filepos message)
  (let loop ((res '()) (i 0))
    (let ((next (po-message-filepos message i)))
      (if next
	  (loop (cons next res) (+ 1 i))
	  (reverse res)))))

(define (po-message-remove-filepos! message position)
  (let ((pointer (po-message-pointer message)))
    (%po-message-remove-filepos! pointer position)))

(define (po-message-add-filepos! message file line)
  (let ((pointer (po-message-pointer message))
	(file-pointer (string->pointer file)))
    (%po-message-add-filepos! pointer file-pointer line)))
