;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Scheme API for header entry API from libgettextpo.
;;

;;; Code:
(define-module (gettext po header)
  #:use-module (gettext po file)
  #:use-module (gettext po internal-api)
  #:use-module (ice-9 regex)
  #:use-module (srfi srfi-9)
  #:use-module (system foreign)
  #:export (<po-header>
	    po-header?
	    po-header-po-file
	    po-header-domain
	    po-header-text

	    po-file-domain-header
	    po-header-field
	    po-header-set-field

	    ;; Utility function
	    po-file-nplurals))

(define-record-type <po-header>
  (make-po-header po-file domain text)
  po-header?
  (po-file po-header-po-file)
  (domain po-header-domain)
  (text po-header-text))

(define* (po-file-domain-header po-file #:key (domain #f))
  (let* ((pointer (po-file-pointer po-file))
	 (domain-ptr (if domain domain %null-pointer))
	 (result (%po-file-domain-header pointer domain-ptr)))
    (and result (not (null-pointer? result))
	 (make-po-header po-file domain (pointer->string result)))))

(define (po-header-field header field)
  (let ((header-str (po-header-text header)))
    (%po-header-field header-str field)))

(define (po-header-set-field header field value)
  (let ((text (po-header-text header))
	(po-file (po-header-po-file header))
	(domain (po-header-domain header)))
    (let ((result (%po-header-set-field text field value)))
     (and result (make-po-header po-file domain result)))))

(define po-file-nplurals
  (let ((nplurals-rx (make-regexp "nplurals=([0-9]+)" regexp/extended)))
    (define (parse-nplurals str)
      (let* ((found (regexp-exec nplurals-rx str))
	     (num (match:substring found 1)))
	(and num (string->number num))))
    (lambda* (po-file #:key (domain #f))
      (let* ((header (po-file-domain-header po-file #:domain domain))
	     (plural-forms (po-header-field header "PluralForms")))
	(or (and plural-forms (parse-nplurals plural-forms))
	    0)))))
