;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; This API covers gettext-po.h, from GNU Gettext sources.
;;

;;; Code:
(define-module (gettext po internal-api)
  #:use-module (gettext po config)
  #:use-module (system foreign)
  #:export (PO_SEVERITY_WARNING
	    PO_SEVERITY_ERROR
	    PO_SEVERITY_FATAL_ERROR

	    %po-file-create
	    %po-file-read
	    %po-file-write
	    %po-file-free
	    %po-file-domains
	    %po-file-domain-header
	    %po-header-field
	    %po-header-set-field

	    %po-message-iterator
	    %po-message-iterator-free
	    %po-message-insert
	    %po-next-message

	    %po-message-create
	    %po-message-msgctxt
	    %po-message-set-msgctxt!
	    %po-message-msgid
	    %po-message-set-msgid!
	    %po-message-msgid-plural
	    %po-message-set-msgid-plural!
	    %po-message-msgstr
	    %po-message-set-msgstr!
	    %po-message-msgstr-plural
	    %po-message-set-msgstr-plural!
	    %po-message-comments
	    %po-message-set-comments!
	    %po-message-extracted-comments
	    %po-message-set-extracted-comments!
	    %po-message-filepos
	    %po-message-add-filepos!
	    %po-message-remove-filepos!
	    %po-message-prev-msgctxt
	    %po-message-set-prev-msgctxt!
	    %po-message-prev-msgid
	    %po-message-set-prev-msgid!
	    %po-message-prev-msgid-plural
	    %po-message-set-prev-msgid-plural!
	    %po-message-is-obsolete?
	    %po-message-set-obsolete!
	    %po-message-is-fuzzy?
	    %po-message-set-fuzzy!
	    %po-message-is-format?
	    %po-message-set-format!
	    %po-message-is-range?
	    %po-message-set-range!

	    %po-filepos-file
	    %po-filepos-start-line

	    %po-format-list
	    %po-format-pretty-name

	    %po-file-check-all
	    %po-message-check-all
	    %po-message-check-format))

;; PO_SEVERITY_*, %po-file-read, %po-file-write, %po-file-domains,
;; %po-header-field, %po-header-set-field, %po-message-is-range,
;; %po-format-list, %po-file-check-all, %po-message-check-all,
;; %po-message-check-format
(dynamic-call "init_gettextpo_extension" (%libgettextpo-guile-lib))

(define-syntax define-cfunc
  (syntax-rules (cdecl)
    ((_ fname (cdecl ret cname (arg . rest)))
     (define fname
       (let* ((str-name (symbol->string cname))
	     (fptr (dynamic-pointer str-name (%libgettextpo-lib))))
	 (pointer->procedure ret fptr (list arg . rest)))))
    ((_ fname (cdecl ret cname ()))
     (define fname
       (let* ((str-name (symbol->string cname))
	      (fptr (dynamic-pointer str-name (%libgettextpo-lib))))
	 (pointer->procedure ret fptr '()))))))

;; Some types for better reading.
(define %cstring '*)
(define %po-file-t '*)
(define %po-message-iterator-t '*)
(define %po-message-t '*)
(define %po-filepos-t '*)

;; Po files
(define-cfunc %po-file-create
  (cdecl %po-file-t 'po_file_create ()))

(define %po-file-free
  (dynamic-pointer "po_file_free" (%libgettextpo-lib)))

;; Header functions
(define-cfunc %po-file-domain-header
  (cdecl %cstring 'po_file_domain_header (%po-file-t %cstring)))

;; Message iterator
(define-cfunc %po-message-iterator
  (cdecl %po-message-iterator-t 'po_message_iterator (%po-file-t %cstring)))

(define %po-message-iterator-free
  (dynamic-pointer "po_message_iterator_free" (%libgettextpo-lib)))

(define-cfunc %po-next-message
  (cdecl %po-message-t 'po_next_message (%po-message-iterator-t)))

(define-cfunc %po-message-insert
  (cdecl void 'po_message_insert (%po-message-iterator-t %po-message-t)))

;; Message
(define-cfunc %po-message-create
  (cdecl %po-message-t 'po_message_create ()))

(define-cfunc %po-message-msgctxt
  (cdecl %cstring 'po_message_msgctxt (%po-message-t)))

(define-cfunc %po-message-set-msgctxt!
  (cdecl void 'po_message_set_msgctxt (%po-message-t %cstring)))

(define-cfunc %po-message-msgid
  (cdecl %cstring 'po_message_msgid (%po-message-t)))

(define-cfunc %po-message-set-msgid!
  (cdecl void 'po_message_set_msgid (%po-message-t %cstring)))

(define-cfunc %po-message-msgid-plural
  (cdecl %cstring 'po_message_msgid_plural (%po-message-t)))

(define-cfunc %po-message-set-msgid-plural!
  (cdecl void 'po_message_set_msgid_plural (%po-message-t %cstring)))

(define-cfunc %po-message-msgstr
  (cdecl %cstring 'po_message_msgstr (%po-message-t)))

(define-cfunc %po-message-set-msgstr!
  (cdecl void 'po_message_set_msgstr (%po-message-t %cstring)))

(define-cfunc %po-message-msgstr-plural
  (cdecl %cstring 'po_message_msgstr_plural (%po-message-t int)))

(define-cfunc %po-message-set-msgstr-plural!
  (cdecl void 'po_message_set_msgstr_plural (%po-message-t int %cstring)))

(define-cfunc %po-message-comments
  (cdecl %cstring 'po_message_comments (%po-message-t)))

(define-cfunc %po-message-set-comments!
  (cdecl void 'po_message_set_comments (%po-message-t %cstring)))

(define-cfunc %po-message-extracted-comments
  (cdecl %cstring 'po_message_extracted_comments (%po-message-t)))

(define-cfunc %po-message-set-extracted-comments!
  (cdecl void 'po_message_set_extracted_comments (%po-message-t %cstring)))

(define-cfunc %po-message-filepos
  (cdecl %po-filepos-t 'po_message_filepos (%po-message-t int)))

(define-cfunc %po-message-remove-filepos!
  (cdecl void 'po_message_remove_filepos (%po-message-t int)))

(define-cfunc %po-message-add-filepos!
  (cdecl void 'po_message_add_filepos (%po-message-t %cstring size_t)))

(define-cfunc %po-message-prev-msgctxt
  (cdecl %cstring 'po_message_prev_msgctxt (%po-message-t)))

(define-cfunc %po-message-set-prev-msgctxt!
  (cdecl void 'po_message_set_prev_msgctxt (%po-message-t %cstring)))

(define-cfunc %po-message-prev-msgid
  (cdecl %cstring 'po_message_prev_msgid (%po-message-t)))

(define-cfunc %po-message-set-prev-msgid!
  (cdecl void 'po_message_set_prev_msgid (%po-message-t %cstring)))

(define-cfunc %po-message-prev-msgid-plural
  (cdecl %cstring 'po_message_prev_msgid_plural (%po-message-t)))

(define-cfunc %po-message-set-prev-msgid-plural!
  (cdecl void 'po_message_set_prev_msgid_plural (%po-message-t %cstring)))

(define-cfunc %po-message-is-obsolete?
  (cdecl int 'po_message_is_obsolete (%po-message-t)))

(define-cfunc %po-message-set-obsolete!
  (cdecl void 'po_message_set_obsolete (%po-message-t int)))

(define-cfunc %po-message-is-fuzzy?
  (cdecl int 'po_message_is_fuzzy (%po-message-t)))

(define-cfunc %po-message-set-fuzzy!
  (cdecl void 'po_message_set_fuzzy (%po-message-t int)))

(define-cfunc %po-message-is-format?
  (cdecl int 'po_message_is_format (%po-message-t %cstring)))

(define-cfunc %po-message-set-format!
  (cdecl void 'po_message_set_format (%po-message-t %cstring int)))

(define-cfunc %po-message-set-range!
  (cdecl void 'po_message_set_range (%po-message-t int int)))

(define-cfunc %po-filepos-file
  (cdecl %cstring 'po_filepos_file (%po-filepos-t)))

(define-cfunc %po-filepos-start-line
  (cdecl size_t 'po_filepos_start_line (%po-filepos-t)))

(define-cfunc %po-format-pretty-name
  (cdecl %cstring 'po_format_pretty_name (%cstring)))
