;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Basic error management.
;;

;;; Code:
(define-module (test-errors)
  #:use-module (gettext po)
  #:use-module (srfi srfi-19)
  #:use-module (srfi srfi-64))

(define %abs-top-srcdir (getenv "abs_top_srcdir"))
(define %tests-data-dir (getenv "tests_data_dir"))
(define %bad-po-format-error-filename
  (string-append %abs-top-srcdir "/" %tests-data-dir "/bad-po-format.po"))
(define %po-file-format-error-filename
  (string-append %abs-top-srcdir "/" %tests-data-dir "/bad-scheme-format.po"))

(define %test-handler (make-throw-message-handler))

(test-begin "errors")

(test-assert "read-error-returns-empty-file"
  (read-po-file %bad-po-format-error-filename))

(test-assert "throw-in-error-handler-returns-false"
  (not (read-po-file %bad-po-format-error-filename
		     #:error-handler %test-handler)))

(let ((exception-key 'test))
  (test-error "throw-in-error-handler-other-exception" exception-key
    (read-po-file %bad-po-format-error-filename
		  #:error-handler (make-throw-message-handler exception-key))))

;; White box test
(let ((exception-key (@@ (gettext po error) %exception-type)))
  (test-error "rethrow-exception-type-from-implementation" exception-key
    (read-po-file %bad-po-format-error-filename
		  #:error-handler %test-handler
		  #:exception-handler %test-handler)))

(let ((po-file (read-po-file %po-file-format-error-filename)))
  (test-assert "error-format-scheme-format-read-ok" po-file)

  (unless po-file
    (test-skip 1))
  (test-group "error-format-scheme-format"
    (let ((messages (po-file-messages po-file)))
      (test-eqv "error-format-scheme-format-num-messages"
	1 (length messages))
      (let ((message (car messages)))
	(test-assert "error-format-scheme-format-file-check-all"
	  (not (po-file-check-all po-file #:error-handler %test-handler)))
	(test-assert "error-format-scheme-format-message-check-format"
	  (not (po-message-check-format message #:error-handler %test-handler)))
	(let* ((file (make-po-file))
	       (iterator (make-po-message-iterator file)))
	  (test-assert "error-format-scheme-format-message-check-all"
	    (not (po-message-check-all message iterator
				       #:error-handler %test-handler))))))))

(test-end "errors")
