;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Iteration tests
;;

;;; Code:
(define-module (test-iteration)
  #:use-module (gettext po)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-19)
  #:use-module (srfi srfi-64))

(test-begin "iteration")

(define %abs-top-srcdir (getenv "abs_top_srcdir"))
(define %tests-data-dir (getenv "tests_data_dir"))
(define %po-file-complete-filename
  (string-append %abs-top-srcdir "/" %tests-data-dir "/complete.po"))
(define %po-file-bad-po-format-filename
  (string-append %abs-top-srcdir "/" %tests-data-dir "/bad-po-format.po"))

(define po-file-complete #f)
(test-group "iteration-read-complete"
  (let ((complete (read-po-file %po-file-complete-filename)))
    (test-assert "iteration-read-complete file read" complete)
    (set! po-file-complete complete)))

(unless po-file-complete
  (test-skip 2))
(test-group "iteration-complete-iterator-constraints"
  (let ((iterator1 (make-po-message-iterator po-file-complete))
	(iterator2 (make-po-message-iterator po-file-complete)))
    (test-assert "iteration-complete-iterator1"
      (po-message-iterator? iterator1))
    (test-assert "iteration-complete-iterator2"
      (po-message-iterator? iterator2))
    (test-assert "iteration-complete-disjoint-id"
      (not (eq? iterator1 iterator2)))
    (test-assert "iteration-complete-disjoint-value"
      (not (po-message-iterator-eqv? iterator1 iterator2)))

    (let ((message1 (po-next-message iterator1))
	  (message2 (po-next-message iterator2)))
      (test-assert "iteration-complete-iterator1-first"	(po-message? message1))
      (test-assert "iteration-complete-iterator2-first"	(po-message? message2))
      (test-equal "iteration-complete-iterator-first-header-1"
	"" (po-message-msgid message1))
      (test-equal "iteration-complete-iterator-first-header-2"
	"" (po-message-msgid message2))
      (test-assert "iteration-complete-iterator-first-same-value"
	(po-message-eqv? message1 message2)))))

(test-group "iteration-complete-messages-constraints"
  (let ((messages (po-file-messages po-file-complete)))
    (test-eqv "iteration-complete-messages-length" 11 (length messages))
    (test-eqv "iteration-complete-messages-type"
      11 (count po-message? messages))))

(define po-file-complete-2 #f)
(test-group "iteration-read-complete-2"
  (let ((complete (read-po-file %po-file-complete-filename)))
    (test-assert "iteration-read-complete-2 file read" complete)
    (set! po-file-complete-2 complete)))

(unless po-file-complete-2
  (test-skip 2))
(test-group "iteration-complete-2-iterator-constraints"
  (let ((iterator1 (make-po-message-iterator po-file-complete))
	(iterator2 (make-po-message-iterator po-file-complete-2)))
    (test-assert "iteration-complete-2-iterator1"
      (po-message-iterator? iterator1))
    (test-assert "iteration-complete-2-iterator2"
      (po-message-iterator? iterator2))
    (test-assert "iteration-complete-2-disjoint-id"
      (not (eq? iterator1 iterator2)))
    (test-assert "iteration-complete-2-disjoint-value"
      (not (po-message-iterator-eqv? iterator1 iterator2)))

    (let ((message1 (po-next-message iterator1))
	  (message2 (po-next-message iterator2)))
      (test-assert "iteration-complete-2-iterator1-first"
	(po-message? message1))
      (test-assert "iteration-complete-2-iterator2-first"
	(po-message? message2))
      (test-assert "iteration-complete-2-iterator-disjoint-id"
      	(not (eq? message1 message2)))
      (test-assert "iteration-complete-2-iterator-disjoint-pointer"
	(not (po-message-eqv? message1 message2)))
      (test-equal "iteration-complete-2-iterator-equal-msgid"
	(po-message-msgid message1) (po-message-msgid message2))
      (test-equal "iteration-complete-2-iterator-equal-msgstr"
	(po-message-msgstr message1) (po-message-msgstr message2))
      (test-equal "iteration-complete-2-iterator-equal-comment"
	(po-message-comments message1) (po-message-comments message2))
      (test-equal "iteration-complete-2-iterator-equal-extracted-comment"
	(po-message-extracted-comments message1)
	(po-message-extracted-comments message2)))))

(test-end "iteration")
