;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Basic PO file writing.
;;

;;; Code:
(define-module (test-write)
  #:use-module (gettext po)
  #:use-module (srfi srfi-19)
  #:use-module (srfi srfi-64)
  #:use-module (system foreign))

(test-begin "write")

(define %abs-top-srcdir (getenv "abs_top_srcdir"))
(define %tests-data-dir (getenv "tests_data_dir"))

(define po-file-complete
  (let ((complete (string-append %abs-top-srcdir "/" %tests-data-dir
				 "/complete.po")))
    (read-po-file complete)))

(unless po-file-complete
  (test-skip 1))
(let ((file (string-append %tests-data-dir "/t-write-complete.po")))
  (when (access? file F_OK) (delete-file file))
  (test-group-with-cleanup "write-complete"
    (test-assert "write-complete-write-file-ok?"
      (write-po-file po-file-complete file))
    (test-assert "write-complete-access?"
      (access? file R_OK))
    (test-assert "write-complete-diff?"
      (system (string-append "diff " file " "
			     (po-file-filename po-file-complete))))
    (when (access? file F_OK) (delete-file file))))

(define po-file-one-msgctxt "Test msctxt")
(define po-file-one-msgid "Test msgid")
(define po-file-one-msgstr "Test msgstr")
(define po-file-one-comments "Test comment\n")
(define po-file-one-extracted-comments "Test extracted comment\n")
(define po-file-one-filepos-file "test.scm")
(define po-file-one-filepos-line 150)
(define po-file-one
  (let ((test-file (make-po-file))
	(message (make-po-message)))
    (po-message-set-msgctxt! message po-file-one-msgctxt)
    (po-message-set-msgid! message po-file-one-msgid)
    (po-message-set-msgstr! message po-file-one-msgstr)
    (po-message-set-comments! message po-file-one-comments)
    (po-message-set-extracted-comments! message po-file-one-extracted-comments)
    (po-message-add-filepos! message po-file-one-filepos-file
			     po-file-one-filepos-line)
    (po-message-append! test-file message)
    test-file))

(let ((file (string-append %tests-data-dir "t-write-one.po")))
  (when (access? file F_OK) (delete-file file))
  (test-group-with-cleanup "write-one-message"
    (define result-file #f)
    (test-assert "write-one-message-result"
      (let ((result (write-po-file po-file-one file)))
	(set! result-file result)
	result))

    (unless result-file
      (test-skip 2))
    (test-group "write-one-message-checks"
      (test-eqv "write-one-message-keep-pointer"
	(pointer-address (po-file-pointer po-file-one))
	(pointer-address (po-file-pointer result-file)))
      (test-equal "write-one-message-keep-filename"
	file (po-file-filename result-file)))

    (test-group "write-one-read-again"
      (define read-file #f)
      (test-assert "write-one-message-read-file"
	(let ((new-result (read-po-file file)))
	  (set! read-file new-result)
	  new-result))

      (unless read-file
	(test-skip 1))
      (test-group "write-one-message-message-content"
        (let ((messages (po-file-messages read-file)))
	  (test-eqv "write-one-message-message-number"
	    1 (length messages))
	  (let ((message (car messages)))
	    (test-equal "write-one-message-msgctxt"
	      po-file-one-msgctxt (po-message-msgctxt message))
	    (test-equal "write-one-message-msgid"
	      po-file-one-msgid (po-message-msgid message))
	    (test-equal "write-one-message-msgstr"
	      po-file-one-msgstr (po-message-msgstr message))
	    (test-equal "write-one-message-comment"
	      po-file-one-comments (po-message-comments message))
	    (test-equal "write-one-message-extracted-comment"
	      po-file-one-extracted-comments
	      (po-message-extracted-comments message))
	    (let ((all-filepos (po-message-all-filepos message)))
	      (test-assert "write-one-message-filepos?" all-filepos)
	      (test-eqv "write-one-message-filepos-num"
		1 (length all-filepos))
	      (let ((filepos (car all-filepos)))
		(test-equal "write-one-message-filepos-file"
		  po-file-one-filepos-file (po-filepos-file filepos))
		(test-eqv "write-one-message-filepos-file"
		  po-file-one-filepos-line (po-filepos-start-line filepos))))))))

    (when (access? file F_OK) (delete-file file))))

(test-end "write")
